from __future__ import (absolute_import, division,print_function)
import maya.cmds as cmds
import maya.api.OpenMaya as om

import os
import json
from xml.etree import ElementTree
from . import rigSettings

##----------------------------------------------------------------------------------------
##editStrings
##----------------------------------------------------------------------------------------
def capitalizeString(sourceString):
    distString = sourceString[0].upper() + sourceString[1:]
    return distString

def addPrefix(prefix,sourceString):    
    if sourceString == "":
        return prefix    
    if prefix != "":
        return prefix + capitalizeString(sourceString)
    else:
        return sourceString

def getSideString(side):
    sideString = ""
    
    if side in ["L","left","Left"]:
        sideString = "_L"

    if side in ["R","right","Right"]:
        sideString = "_R"
    
    return sideString

def geneParamName(useParamPrefix,paramName,section,side,prefix):
    
    if useParamPrefix == "none" or useParamPrefix == "":
        return paramName
    
    elif useParamPrefix == "section":    
        return addPrefix(section,paramName)
        
    elif useParamPrefix == "section+side":        
        return addPrefix(section,paramName) + getSideString(side)

    elif useParamPrefix == "prefix":
        return addPrefix(prefix,paramName)

    elif useParamPrefix == "prefix+side":
        return addPrefix(prefix,paramName) + getSideString(side)
    
##----------------------------------------------------------------------------------------
##editFiles
##----------------------------------------------------------------------------------------
def readJSON(filePath,defaultDict = None):
    importDict = {}

    if os.path.exists(filePath) == True and os.path.isfile(filePath):
        with open(filePath) as f:
            importDict = json.load(f)	

    if defaultDict == None:
        return importDict

    else:
        for key in list(defaultDict.keys()):
            if key not in list(importDict.keys()):
                importDict[key] = defaultDict[key]

    return importDict

def writeJSON(filePath,valueDict,indent = 4):	
    with open(filePath, 'w') as f:
        json.dump(valueDict, f, indent = indent)

##----------------------------------------------------------------------------------------
##editAttrs
##----------------------------------------------------------------------------------------
def getFlattenAttr(target,attrs = []):
    targetAttrs = []
    for attr in attrs:
        if cmds.attributeQuery(attr,node = target,exists =True) ==False:
            if cmds.objExists(target + "." + attr):
                targetAttrs.append(attr)
            continue

        childrenAttrs = cmds.attributeQuery(attr,node = target,listChildren =True)        
        
        if childrenAttrs == None or len(childrenAttrs) == 0:
            targetAttrs.append(attr)
        else:
            targetAttrs.extend(childrenAttrs)

    return targetAttrs

def setAttrState(targets,attrs,lock,hide,keyable):
    for target in targets:
        targetAttrs = getFlattenAttr(target,attrs)
        for attr in targetAttrs:
            if cmds.objExists(target + "." + attr) == False:
                continue
            #lock
            cmds.setAttr(target + '.'+ attr, lock = lock)

            if hide:
                cmds.setAttr(target + '.'+ attr,  channelBox = False, keyable = False)
            else :
                cmds.setAttr(target + '.'+ attr, channelBox = True, keyable = True)
                cmds.setAttr(target + '.'+ attr, keyable = keyable)

def connectAttrZip(source,target,sourceAttrs,targetAttrs,f =True):
    sourceAttrs = getFlattenAttr(source,sourceAttrs)
    targetAttrs = getFlattenAttr(target,targetAttrs)
    
    for sourceAttr,targetAttr in zip(sourceAttrs,targetAttrs):           
        cmds.connectAttr(source + "." + sourceAttr,target + "." + targetAttr, f =f)

def connectAttrMerge(source,target,attr,mode = "multi"):
    curInput = cmds.listConnections(target + "."+attr, s=True,d =False, p =True,skipConversionNodes=True) 
    
    if curInput != None:
        multiVis = cmds.createNode("floatMath")

        if mode == "multi":
            cmds.setAttr(multiVis + ".operation",2)

        elif mode == "add":
            cmds.setAttr(multiVis + ".operation",0)

        cmds.connectAttr(curInput[0],multiVis + ".floatA")
        cmds.connectAttr(source,multiVis + ".floatB")
        cmds.connectAttr(multiVis + ".outFloat",target + "."+attr,f =True)

    else:
        cmds.connectAttr(source,target + "."+attr)


def listAttrs(node,keyable,unkeyable,longName = False):
    attrs = []

    unKeyableAttrs = cmds.listAttr(node,channelBox =True) or []
    keyableAttrs = cmds.listAttr(node,keyable =True) or []
    
    if keyable:

        for attr in keyableAttrs:
            attrName = cmds.attributeQuery(attr, node=node, longName=longName,  shortName = not longName)
            attrs.append(attrName)

    if unkeyable:
        for attr in unKeyableAttrs:
            attrName = cmds.attributeQuery(attr, node=node, longName=longName,  shortName = not longName)
            attrs.append(attrName)

    return attrs

def getTransformLimit(node):
    limitDict = {}

    for attr in ["Trans","Rot","Scale"]:
        for axis in "XYZ":
            if cmds.getAttr(node + ".min"+attr+axis+"LimitEnable"):
                limitDict["min"+attr+axis+"LimitEnable"] = True
                limitDict["min"+attr+axis+"Limit"] = cmds.getAttr(node + ".min"+attr+axis+"Limit")
            
            if cmds.getAttr(node + ".max"+attr+axis+"LimitEnable"):
                limitDict["max"+attr+axis+"LimitEnable"] = True
                limitDict["max"+attr+axis+"Limit"] = cmds.getAttr(node + ".max"+attr+axis+"Limit")

    return limitDict
##----------------------------------------------------------------------------------------
##listTransformNodes
##----------------------------------------------------------------------------------------
def getCurHierarchy(target,topNode = None):
    hierarchyList = []
    hierarchyList.append(target)

    if target == topNode:
        return hierarchyList

    reachTop = False    
    while reachTop == False:
        parent = cmds.listRelatives(hierarchyList[-1], p =True)
        
        if parent == None:
            reachTop = True
            continue

        elif parent[0] == topNode:
            reachTop = True

        hierarchyList.append(parent[0])
    hierarchyList.reverse()

    return hierarchyList

def listSetMembers(setName,nodeTypes,allHierarchy):
    if cmds.objExists(setName) ==False:
        return []

    allMembers = cmds.sets(setName,q=True)
    memberSets = cmds.ls(allMembers, type = "objectSet") or []
    memberNodes = []

    for nodeType in nodeTypes:
        nodes = cmds.ls(allMembers, type = nodeType) or []
        memberNodes = list(set(memberNodes) | set(nodes))

    if allHierarchy:
        ##setMemberが objectSetの場合、掘り下げる
        for memberSet in memberSets:
            memberNodes.extend(listSetMembers(memberSet,nodeTypes,allHierarchy))
    
    return memberNodes

def listIKJoints():    
    ikHandles = cmds.ls(type = "ikHandle") or []

    IKJoints = []
    for ikHandle in ikHandles:
        startJoint = cmds.listConnections(ikHandle + ".startJoint",s=True, d=False) or []
        endEffector = cmds.listConnections(ikHandle + ".endEffector",s=True, d=False) or []
        endJoint = cmds.listConnections(endEffector[0] + ".offsetParentMatrix",s=True, d=True) or []
        jointChain = getCurHierarchy(endJoint,startJoint)
        IKJoints = list(set(IKJoints) | set(jointChain))

    return IKJoints

def listNodes(topNode,ctrlSets):
    ##階層下全てのtransformノードを抽出
    allTransformNodes = cmds.listRelatives(topNode,ad =True, type ="transform") or []
    allTransformNodes.append(topNode)

    ##コンストレインノードを抽出
    constraintNodes = cmds.listRelatives(topNode,ad =True, type ="constraint") or []
    otherNodes = list(set(allTransformNodes) - set(constraintNodes))

    ##IKhandleで制御されているjointはlockしてしまうと正しく動かなくなるので、別途抽出    
    IKJoints = listIKJoints()
    otherNodes = list(set(otherNodes) - set(IKJoints))

    ##ctrlsetsに登録されているノードを抽出
    ctrlNodes = listSetMembers(ctrlSets,["transform"],True)
    otherNodes = list(set(otherNodes) - set(ctrlNodes))
    
    nodeDict = {
                "ctrl":ctrlNodes,
                "IKJoints":IKJoints,
                "constraint":constraintNodes,
                "other":otherNodes
    }

    return nodeDict


##----------------------------------------------------------------------------------------
##edit Transform
##----------------------------------------------------------------------------------------
def getPosition(target):
    nodeType = cmds.nodeType(target)
    
    if nodeType == "joint" or nodeType == "transform" or nodeType == "ikHandle" or nodeType == "place3dTexture":
        # return cmds.xform(target , q =True , ws = True, t =True)
        return cmds.xform(target , q =True , ws = True, rotatePivot =True)

    elif nodeType == "mesh" or nodeType == "nurbsCurve" or nodeType == "nurbsSurface" or nodeType == "lattice":
        return cmds.xform(target , q =True , ws = True, t =True)

def getRotation(target):
    return cmds.xform(target , q =True , ws = True, ro =True)

def snapPosition(source,target):
    destPosition = getPosition(source)
    cmds.xform(target, ws =True, t =destPosition)

    return

def snapRotation(source,target):
    rotation = getRotation(source)
    cmds.xform(target, ws =True, ro = rotation)

    if cmds.nodeType(target) == "joint":
        cmds.makeIdentity(target,apply = True,r =1)
    return

def getDagNode(target):    
    try:
        sellist = om.MGlobal.getSelectionListByName(target)
        return sellist.getDagPath(0)
    except:
        return None
    
def getShapeFn(dagPath):
    FnShape = None

    if dagPath.hasFn(om.MFn.kMesh):
        FnShape = om.MFnMesh(dagPath)        

    elif dagPath.hasFn(om.MFn.kNurbsSurface):
        FnShape = om.MFnNurbsSurface(dagPath)

    elif dagPath.hasFn(om.MFn.kNurbsCurve):
        FnShape = om.MFnNurbsCurve(dagPath)

    return FnShape

def reParent(target,newParent):
    curParent = cmds.listRelatives(target,p=True)
    if curParent == None:
        cmds.parent(target,newParent,a =True)
    
    elif curParent[0] != newParent:
        cmds.parent(target,newParent,a =True)
        
    return 

def insertHierarchy(target,insertNode):
    curParent = cmds.listRelatives(target,p=True)

    if curParent != None:
        reParent(insertNode,curParent[0])
    
    reParent(target,insertNode)
    return

def insertOffsetNode(nodeType,target,suffix):
    offsetNode = cmds.createNode(nodeType, name = target + suffix)
    
    snapPosition(target,offsetNode)
    snapRotation(target,offsetNode)
    insertHierarchy(target,offsetNode)

    if nodeType == "joint":
        cmds.setAttr(offsetNode + ".drawStyle",2)
        
    return offsetNode    
##----------------------------------------------------------------------------------------
##for matrix
##----------------------------------------------------------------------------------------
def getJointOrientMatrix(target,inverse = False):
    composeMatrix = cmds.createNode("composeMatrix")
    cmds.setAttr(composeMatrix + ".inputRotate", *cmds.getAttr(target + ".jointOrient")[0])    
    returnMatrix = cmds.getAttr(composeMatrix + ".outputMatrix")    
    inversematrix = cmds.createNode("inverseMatrix")
    cmds.connectAttr(composeMatrix + ".outputMatrix" , inversematrix + ".inputMatrix")
    returnInvMatrix = cmds.getAttr(inversematrix + ".outputMatrix")
    cmds.delete(composeMatrix)
    if inverse:
        return returnInvMatrix
    return returnMatrix

def createHierarchyMatrix(target,spaceParent):
    hierarchyList = getCurHierarchy(target,spaceParent)
    hierarchyList.reverse()
    multmatrix = cmds.createNode("multMatrix")
    
    if len(hierarchyList) == 1:        
        cmds.connectAttr(hierarchyList[0] + ".matrix", multmatrix + ".matrixIn[0]")
        
        if cmds.objExists(hierarchyList[0] + ".offsetParentMatrix"):
            cmds.connectAttr(hierarchyList[0] + ".offsetParentMatrix", multmatrix + ".matrixIn[1]")
    else:
        ii = 0
        for i in range(0,len(hierarchyList)-1):            
            cmds.connectAttr(hierarchyList[i] + ".matrix", multmatrix + ".matrixIn["+str(ii)+"]")
            ii = ii + 1

            if cmds.objExists(hierarchyList[i] + ".offsetParentMatrix"):
                cmds.connectAttr(hierarchyList[i] + ".offsetParentMatrix", multmatrix + ".matrixIn["+str(ii)+"]")
                ii = ii + 1
    return multmatrix

def createMultMatrix(source,target,sourceParent,targetParent,offset):
    if offset ==True:
        offsetTemp = cmds.createNode("transform",name = target + "offsettemp")
        snapPosition(target,offsetTemp)
        snapRotation(target,offsetTemp)        
        cmds.parent(offsetTemp,source,a = True)
        
        multmatrix = createHierarchyMatrix(offsetTemp,sourceParent)

        cmds.disconnectAttr(offsetTemp+ ".matrix", multmatrix + ".matrixIn["+str(0)+"]")
        cmds.setAttr(multmatrix + ".matrixIn["+str(0)+"]", *cmds.getAttr(offsetTemp+ ".matrix"),type = "matrix")
        
        if cmds.objExists(offsetTemp + ".offsetParentMatrix"):
            cmds.disconnectAttr(offsetTemp+ ".offsetParentMatrix", multmatrix + ".matrixIn["+str(1)+"]")
            cmds.setAttr(multmatrix + ".matrixIn["+str(1)+"]", *cmds.getAttr(offsetTemp+ ".offsetParentMatrix"),type = "matrix")
        
        cmds.delete(offsetTemp)
    
    else:
        multmatrix = createHierarchyMatrix(source,sourceParent)

    if targetParent != "":
        hierarchyList = getCurHierarchy(target,targetParent)
        hierarchyList = hierarchyList[1:-1]
        latestIndex = cmds.getAttr(multmatrix + ".matrixIn",mi =True)[-1]

        for i in range(0,len(hierarchyList)):
            index = latestIndex + 1 + i
            cmds.connectAttr(hierarchyList[i] + ".inverseMatrix",multmatrix + ".matrixIn["+str(index)+"]")
    
    return multmatrix

def addOffsetJointOrientMatrix(output,target):
    rotOutput = cmds.createNode("decomposeMatrix")
    offsetMatrix = cmds.createNode("multMatrix")
    cmds.connectAttr(output, offsetMatrix + ".matrixIn[0]")
    
    if cmds.listRelatives(target,parent =True) != None:
        parentScaleMatrix = cmds.createNode("composeMatrix")
        parent = cmds.listRelatives(target,parent =True)[0]
        connectAttrZip(parent,parentScaleMatrix,["scale"],["inputScale"],f =True)
        cmds.connectAttr(parentScaleMatrix + ".outputMatrix", offsetMatrix + ".matrixIn[1]")

    ##jointOrient
    jointOrientInvMatrix = getJointOrientMatrix(target,True)
    cmds.setAttr(offsetMatrix + ".matrixIn[2]", jointOrientInvMatrix, type = "matrix")

    cmds.connectAttr(offsetMatrix + ".matrixSum",rotOutput + ".inputMatrix")
    connectAttrZip(rotOutput,target,["outputRotate"],["rotate"],f =True)

    return rotOutput

def matrixConstraint(source,target,sourceParent,targetParent,attrs = ["t","r"],offset = False):
    multmatrix= createMultMatrix(source,target,sourceParent,targetParent,offset)
    outMatrix = cmds.createNode("decomposeMatrix")
    cmds.connectAttr(multmatrix + ".matrixSum", outMatrix + ".inputMatrix")

    if "t" in attrs:
        connectAttrZip(outMatrix,target,["outputTranslate"],["translate"],f =True)

    if "r" in attrs:
        if cmds.nodeType(target) == "joint":
            addOffsetJointOrientMatrix(multmatrix + ".matrixSum",target)

        else:
            connectAttrZip(outMatrix,target,["outputRotate"],["rotate"],f =True)

    return multmatrix,outMatrix

##----------------------------------------------------------------------------------------
##for vector
##----------------------------------------------------------------------------------------
def stringToVector(axis):
    if axis == "x":
        return om.MVector.kXaxisVector 
    elif axis == "y":
        return om.MVector.kYaxisVector 
    elif axis == "z":
        return om.MVector.kZaxisVector 

    elif axis == "-x":
        return om.MVector.kXnegAxisVector
    elif axis == "-y":
        return om.MVector.kYnegAxisVector
    elif axis == "-z":
        return om.MVector.kZnegAxisVector
    
##----------------------------------------------------------------------------------------
##for controller
##----------------------------------------------------------------------------------------
def getCVPositions(target,space = om.MSpace.kWorld):
    dagPath = getDagNode(target)
    FnShape = getShapeFn(dagPath)
    
    cvPositions = []
    if FnShape == None:
        return cvPositions

    for position in FnShape.cvPositions(space):
        cvPositions.append(list(position)[:3])

    return cvPositions

def readCurveShape(target,space = "world"):
    if space == "world":
        space = om.MSpace.kWorld
    elif space == "object":
        space = om.MSpace.kObject

    cvPositions = getCVPositions(target,space = space)
    dagPath = getDagNode(target)
    
    form = None
    knots = None
    degree = None
    FnShape = getShapeFn(dagPath)
    if FnShape != None:
        form = FnShape.form
        knots = list(FnShape.knots())
        degree = FnShape.degree 
        
    return cvPositions,degree,form,knots

def getShapeColorGRB(target):
    shapes = cmds.listRelatives(target,s =True)
    rgb = [0.0,0.0,0.0]

    if shapes == None:
        return rgb

    if cmds.getAttr(shapes[0] + '.useObjectColor') == 2:
        rgb = cmds.getAttr(shapes[0] + '.wireColorRGB')[0]

    return rgb

def setShapeColorGRB(target,RGB):
    shapes = cmds.listRelatives(target,s =True) or []

    for shape in shapes:		
        cmds.setAttr(shape + '.useObjectColor', 2)
        cmds.setAttr(shape + '.wireColorRGB', RGB[0],RGB[1],RGB[2], type = 'double3')

def addWireSettingKeep(target):
    shapes = cmds.listRelatives(target,s =True) or []
    if len(shapes) == 0:
        return

    wireColor = getShapeColorGRB(target)

    if cmds.objExists(target + ".wireColorKeep") == False:
        cmds.addAttr(target, ln = "wireColorKeep",at = "compound",numberOfChildren = 3)
        cmds.addAttr(target, ln = "wireColorKeepR",at = "float", p = "wireColorKeep",dv = wireColor[0], min = 0.0,max = 1.0)
        cmds.addAttr(target, ln = "wireColorKeepG",at = "float", p = "wireColorKeep",dv = wireColor[1], min = 0.0,max = 1.0)
        cmds.addAttr(target, ln = "wireColorKeepB",at = "float", p = "wireColorKeep",dv = wireColor[2], min = 0.0,max = 1.0)

    for shape in shapes:		    
        cmds.setAttr(target + ".wireColorKeepR",l =False)
        cmds.setAttr(target + ".wireColorKeepG",l =False)
        cmds.setAttr(target + ".wireColorKeepB",l =False)
        
        cmds.connectAttr(target + ".wireColorKeepR",shape + ".wireColorR")
        cmds.connectAttr(target + ".wireColorKeepG",shape + ".wireColorG")
        cmds.connectAttr(target + ".wireColorKeepB",shape + ".wireColorB")

def setWireFrameColor(targets,color,tone = 1.0):
    RGB = rigSettings.COLOR_DICT[color]    
    RGB = [RGB[0]*tone,RGB[1]*tone,RGB[2]*tone]

    for target in targets:
        setShapeColorGRB(target,RGB)

def generateKnot(degree,cvNum):
    knot = om.MDoubleArray()
    
    if degree == 1:
        for i in range(0,cvNum):
            knot.append(i)
    
    elif degree == 2:		
        knotCount = 0
                
        for i in range(0,cvNum+1):
            knot.append(knotCount)
        
            if i > 0 and i < (cvNum-1):
                knotCount = knotCount +1
    
    elif degree == 3:
        knotCount = 0
                
        for i in range(0,cvNum+2):
            knot.append(knotCount)
    
            if i > 1 and i < (cvNum-1):
                knotCount = knotCount +1
    
    return list(knot)

def createCurveShape(cvPositions,degree,form,knots = None,parent = None,editPoint = False):
    if knots == None:
        knots = generateKnot(degree,len(cvPositions))

    periodic = False
    if form == 3:
        periodic = True
    
    if editPoint ==False:
        curve = cmds.curve(point = cvPositions, knot = knots, degree = degree,periodic = periodic)
    else:
        curve = cmds.curve(ep = cvPositions, knot = knots, degree = degree,periodic = periodic)
    
    if parent != None:        
        shape = cmds.listRelatives(curve, type = "shape")[0]

        if cmds.objExists(parent) == False:
            cmds.createNode("transform",name = parent)

        cmds.parent(shape,parent,s=True,r=True)
        cmds.rename(shape,parent + "Shape")
        cmds.delete(curve)
        
        curve = parent

    return curve

def createCtrlNode(nodeName,shapeName,nodeType,size = 1.0,position = None,rotation =None):
    transformNode = cmds.createNode(nodeType,name = nodeName)
    
    transformNode = createCurveShape(
                                        rigSettings.CURVE_SHAPE_DICT[shapeName]["cvs"],
                                        rigSettings.CURVE_SHAPE_DICT[shapeName]["degree"],
                                        rigSettings.CURVE_SHAPE_DICT[shapeName]["form"],
                                        rigSettings.CURVE_SHAPE_DICT[shapeName]["knots"],parent = transformNode)

    transformNode = cmds.rename(transformNode,nodeName,ignoreShape =False)
    cmds.xform(transformNode, scale = [size,size,size], ws = True)
    cmds.makeIdentity(transformNode, a =True, r=True, s =True)

    if position != None:
        snapPosition(position,transformNode)

    if rotation != None:
        snapRotation(rotation,transformNode)

    if nodeType == "joint":
        cmds.setAttr(transformNode + ".drawStyle",2)
        setAttrState([transformNode],["radius"],lock=False,hide=True,keyable=False)

    for shape in cmds.listRelatives(transformNode,type = "shape"):
        cmds.setAttr(shape + ".ihi",False)

    return transformNode

def createHierarchy(hierarchyLongName,nodeType,force):
    
    hierarchy = hierarchyLongName.split("|")

    for i in range(0,len(hierarchy)):
        nodeName = hierarchy[i]

        if cmds.objExists(nodeName) == False:
            if nodeType == "objectSet":
                cmds.sets(name = nodeName,empty =True)

            else:
                cmds.createNode(nodeType,name = nodeName)

        if i != 0 and force:
            if nodeType == "objectSet":
                curMember = cmds.sets(hierarchy[i-1],q=True)
                if nodeName not in curMember:
                    cmds.sets(nodeName,forceElement = hierarchy[i-1])

            else:
                reParent(nodeName,hierarchy[i-1])

    return


def addToSet(setLongName,nodes):
    createHierarchy(setLongName,"objectSet",True)

    setName = setLongName.split("|")[-1]

    if cmds.objExists(setName) == False:
        cmds.sets(name = setName, empty=True)

    cmds.sets(nodes,e=True,add = setName)

##----------------------------------------------------------------------------------------
##setup
##----------------------------------------------------------------------------------------
def createSectionSetupGP(section,parent):
    if cmds.objExists(section + "Setup_gp") ==False:
        transformNode = cmds.createNode("transform",name = section + "Setup_gp")
        cmds.parent(transformNode,parent,a =True)

    return section + "Setup_gp"

def createSetupSpace(section,prefix,side,parentJoint):
    sideString = getSideString(side)    
    setupSpace = section + "_" + prefix + sideString + "_setupSpace"

    if cmds.objExists(setupSpace) == False:
        nodeType = "transform"
        
        if parentJoint != None:
            nodeType = cmds.nodeType(parentJoint)

        setupSpace = cmds.createNode(nodeType, name = setupSpace)

        if parentJoint != None:
            connectAttrZip(parentJoint,setupSpace,["scale"],["scale"],f =True)
    
    return setupSpace

def createParentProxy(source,targetParent,attrs = ["t","r","s"]):
    nodeType = cmds.nodeType(source)
    
    if cmds.objExists(source + "_proxy"):
        return source + "_proxy"

    proxyNode = cmds.createNode(nodeType,name = source + "_proxy")
    snapPosition(source,proxyNode)
    snapRotation(source,proxyNode)

    if nodeType == "joint":
        cmds.setAttr(proxyNode + ".drawStyle",2)

    cmds.parent(proxyNode,targetParent,a =True)
    cmds.parentConstraint(source,proxyNode,mo =False)

    if "s" in attrs:
        connectAttrZip(source,proxyNode,["scale"],["scale"],f =True)

    return proxyNode
##----------------------------------------------------------------------------------------
##skincluster
##----------------------------------------------------------------------------------------
def getSkinCluster(node):
    history = cmds.listHistory(node,pruneDagObjects =True)    
    skinNode = cmds.ls(history,type = "skinCluster") or None
    
    if skinNode != None:
        return skinNode[0]

    return None

def removeUnuseInfs(skinNode):
    allInf =  cmds.skinCluster(skinNode,q =True, influence =True)
    weightedInf= cmds.skinCluster(skinNode,q =True, weightedInfluence =True)
    removeInf = list(set(allInf) - set(weightedInf))

    for inf in removeInf:
        cmds.skinCluster(skinNode,e =True, removeInfluence = inf)

def exportSkinWeight(dirPath,node,ext = "xml",cleanup = True,makeDir = True):
    if cmds.nodeType(node) != "transform":
        return
    
    ##skinClusterノードを取得
    skinNode = getSkinCluster(node)
    if skinNode == None:
        return

    ##wieghtが振られていないinfluenceを除去
    if cleanup:
        removeUnuseInfs(skinNode)

    ##skinClsuter名がユニークになるように編集
    if skinNode != node + rigSettings.SUFFIX_DICT["skinCluster"]:
        skinNode = cmds.rename(skinNode, node + rigSettings.SUFFIX_DICT["skinCluster"])
    
    fileName = node + "_skinWeight." + ext
    
    if os.path.exists(dirPath) ==False:
        if makeDir:
            os.makedirs(dirPath)
        else:
            print("do not exists directory for export")
            return None
        
    cmds.deformerWeights(fileName, path = dirPath, deformer = skinNode, format = ext.upper(),ex = True)

    return fileName

def createSkinCluster(target,joints,nodeName = None):
    if nodeName == None:
        nodeName = target + rigSettings.SUFFIX_DICT["skinCluster"]

    if cmds.objExists(nodeName) ==False:
        ##細かいオプションが存在しますが、今回はシンプルな構成にしておきます
        ## skinMethod = リニア
        ## normalizeWeights = True
        cmds.skinCluster(joints,target,skinMethod = 0,normalizeWeights = 1,toSelectedBones =True,name = nodeName)[0]
        
    return nodeName

def getInfluence(skinNode):
    infs = cmds.skinCluster(skinNode, q =True, inf =True)    
    return infs

def addInfluence(target,joints):
    skinNode = getSkinCluster(target)

    if skinNode == None:
        skinNode = createSkinCluster(target,joints)

    curInfs = getInfluence(skinNode)
    toAddInfs = list(set(joints) - set(curInfs))
    
    for inf in toAddInfs:
        cmds.skinCluster(skinNode, e =True, ai = inf, weight = 0)

    return skinNode

def importSkinWeight(path,fileName,overWrite=False):
    allInfsDict = {}
    
    ##XML　からバインド情報を取得
    tree = ElementTree.parse(path + fileName)
    root = tree.getroot()    
    for e in root.findall('weights'):
        inf = e.get("source")
        shape = e.get("shape")
        node = e.get("deformer")

        if node in allInfsDict.keys():
            allInfsDict[node]["infs"].append(inf)
        else:
            allInfsDict[node] = {
                                    "infs":[inf],
                                    "shape":shape
            }

    for skinNode in allInfsDict.keys():
        allInfs = allInfsDict[node]["infs"]
        shape = allInfsDict[node]["shape"]

        ##バインド済みの場合
        if overWrite == False and getSkinCluster(shape) != None:
            continue

        ##skinClusterが存在しなければ作成し、インフルエンスを追加する
        skinNode = addInfluence(shape,allInfs)

        if len(allInfs) != 1:
            cmds.deformerWeights(fileName, path = path, deformer = skinNode, method = "index",im = True)
            cmds.skinCluster(skinNode,e =True,forceNormalizeWeights = True)